@@ -75,25 +75,15 @@ class CodeEvaluator:
           separately for proper functionality.
 
         """
-        self.globals = {
-            "MAX": np.maximum,
-            "MIN": np.minimum,
-            "ABS": np.abs,
-            "QUANTILE": np.quantile,
-            "SUM": np.sum,
-            "MEAN": np.mean,
-            "STD": np.std,
-            "max": np.maximum,
-            "min": np.minimum,
-            "abs": np.abs,
-            "quantile": np.quantile,
-            "sum": np.sum,
-            "mean": np.mean,
-            "std": np.std,
-            "np": np,
-            "nan": np.nan,
-        }
         self.logger = logging.getLogger(__name__)
+        def _mean(*args):
+            return np.mean(args[0])
+        
+        def _std(*args):
+            return np.std(args[0])
+
+        def _quantile(*args):
+            return np.quantile(args[0], args[1])
 
         # general tolerance function
         def _tol(value, direction=bool, column=None):
@@ -173,33 +163,32 @@ class CodeEvaluator:
             ):
                 return left_side == right_side
             else:
+                min_left = np.minimum(left_side_pos, left_side_neg)
+                max_left = np.maximum(left_side_pos, left_side_neg)
+                min_right = np.minimum(right_side_pos, right_side_neg)
+                max_right = np.maximum(right_side_pos, right_side_neg)
+                # explicitly check level to avoid operations if not in debug
                 if logging.DEBUG == logging.root.level:
-                    lhs = [
-                        (str(a), str(b))
-                        for a, b in zip(
-                            np.minimum(left_side_pos, left_side_neg),
-                            np.maximum(left_side_pos, left_side_neg),
-                        )
-                    ]
-                    rhs = [
-                        (str(a), str(b))
-                        for a, b in zip(
-                            np.minimum(right_side_pos, right_side_neg),
-                            np.maximum(right_side_pos, right_side_neg),
-                        )
-                    ]
                     self.logger.debug("Evaluating equality:")
-                    for idx in range(len(lhs)):
+                    if hasattr(min_left, "__iter__") and hasattr(max_left, "__iter__"):
+                        lhs = [(str(a), str(b)) for a, b in zip(min_left, max_left)]
+                        for idx in range(len(lhs)):
+                            self.logger.debug("  LHS: " + str(lhs[idx]))
+                    else:
                         self.logger.debug(
-                            "  LHS: " + str(lhs[idx]) + ", RHS: " + str(rhs[idx])
+                            "  LHS: " + str(min_left) + ", " + str(max_left)
                         )
-                return (
-                    np.maximum(left_side_pos, left_side_neg)
-                    >= np.minimum(right_side_pos, right_side_neg)
-                ) & (
-                    np.minimum(left_side_pos, left_side_neg)
-                    <= np.maximum(right_side_pos, right_side_neg)
-                )
+                    if hasattr(min_right, "__iter__") and hasattr(
+                        max_right, "__iter__"
+                    ):
+                        rhs = [(str(a), str(b)) for a, b in zip(min_right, max_right)]
+                        for idx in range(len(rhs)):
+                            self.logger.debug("  RHS: " + str(rhs[idx]))
+                    else:
+                        self.logger.debug(
+                            "  RHS: " + str(min_right) + ", " + str(max_right)
+                        )
+                return (max_left >= min_right) & (min_left <= max_right)
 
         def _unequal(
             left_side,
@@ -234,35 +223,34 @@ class CodeEvaluator:
             ):
                 return left_side != right_side
             else:
+                min_left = np.minimum(left_side_pos, left_side_neg)
+                max_left = np.maximum(left_side_pos, left_side_neg)
+                min_right = np.minimum(right_side_pos, right_side_neg)
+                max_right = np.maximum(right_side_pos, right_side_neg)
+                # explicitly check level to avoid operations if not in debug
                 if logging.DEBUG == logging.root.level:
-                    lhs = [
-                        (str(a), str(b))
-                        for a, b in zip(
-                            np.minimum(left_side_pos, left_side_neg),
-                            np.maximum(left_side_pos, left_side_neg),
-                        )
-                    ]
-                    rhs = [
-                        (str(a), str(b))
-                        for a, b in zip(
-                            np.minimum(right_side_pos, right_side_neg),
-                            np.maximum(right_side_pos, right_side_neg),
-                        )
-                    ]
-                    self.logger.debug("Evaluating inequality:")
-                    for idx in range(len(lhs)):
+                    self.logger.debug("Evaluating equality:")
+                    if hasattr(min_left, "__iter__") and hasattr(max_left, "__iter__"):
+                        lhs = [(str(a), str(b)) for a, b in zip(min_left, max_left)]
+                        for idx in range(len(lhs)):
+                            self.logger.debug("  LHS: " + str(lhs[idx]))
+                    else:
                         self.logger.debug(
-                            "  LHS: " + str(lhs[idx]) + ", RHS: " + str(rhs[idx])
+                            "  LHS: " + str(min_left) + ", " + str(max_left)
                         )
-                return (
-                    np.minimum(left_side_pos, left_side_neg)
-                    < np.maximum(right_side_pos, right_side_neg)
-                ) | (
-                    np.maximum(left_side_pos, left_side_neg)
-                    > np.minimum(right_side_pos, right_side_neg)
-                )
+                    if hasattr(min_right, "__iter__") and hasattr(
+                        max_right, "__iter__"
+                    ):
+                        rhs = [(str(a), str(b)) for a, b in zip(min_right, max_right)]
+                        for idx in range(len(rhs)):
+                            self.logger.debug("  RHS: " + str(rhs[idx]))
+                    else:
+                        self.logger.debug(
+                            "  RHS: " + str(min_right) + ", " + str(max_right)
+                        )
+                return ~((max_left >= min_right) & (min_left <= max_right))
 
-        def _multiply(a_pos, a_neg, b_pos, b_neg, direction: str):
+        def _mul(a_pos, a_neg, b_pos, b_neg, direction: str):
             """
             Perform multiplication based on the given direction and return the maximum or minimum result
             based on the values of a+, a-, b+, b-.
@@ -295,7 +283,7 @@ class CodeEvaluator:
                     axis=1,
                 ).min(axis=1)
 
-        def _divide(a_pos, a_neg, b_pos, b_neg, direction: str):
+        def _div(a_pos, a_neg, b_pos, b_neg, direction: str):
             """
             Perform division based on the given direction and return the maximum or minimum result
             based on the values of a+, a-, b+, b-.
@@ -328,11 +316,52 @@ class CodeEvaluator:
                     axis=1,
                 ).min(axis=1)
 
+        def _corr(
+            key: str,
+            *columns,
+        ):
+            """
+            # (sum r,s Corr (r,s) * column (r) * column (s)
+            """
+            if key not in list(self.matrices.keys()):
+                logging.error(
+                    'Matrix key "'
+                    + key
+                    + '" is not in predefined matrices dictionary of parameters.'
+                )
+            m = self.matrices[key]
+            result = pd.Series([0] * len(columns[0]))
+            for r in range(len(m)):
+                for s in range(len(m)):
+                    result += m[r][s] * columns[r] * columns[s]
+            return result
+
+        self.globals = {
+            "MAX": np.maximum,
+            "MIN": np.minimum,
+            "ABS": np.abs,
+            "QUANTILE": _quantile,
+            "SUM": np.sum,
+            "MEAN": _mean,
+            "STD": _std,
+            "max": np.maximum,
+            "min": np.minimum,
+            "abs": np.abs,
+            "quantile": _quantile,
+            "sum": np.sum,
+            "mean": _mean,
+            "std": _std,
+            "np": np,
+            "nan": np.nan,
+        }
+
+
         self.globals["_tol"] = _tol
         self.globals["_equal"] = _equal
         self.globals["_unequal"] = _unequal
-        self.globals["_multiply"] = _multiply
-        self.globals["_divide"] = _divide
+        self.globals["_mul"] = _mul
+        self.globals["_div"] = _div
+        self.globals["_corr"] = _corr
 
     def set_params(self, params):
         """
@@ -365,6 +394,11 @@ class CodeEvaluator:
                     raise Exception(
                         "No spaces allowed in keys of tolerance definition."
                     )
+        matrices = self.params.get("matrices", None)
+        if matrices is not None:
+            self.matrices = dict()
+            for key, value in matrices.items():
+                self.matrices[key] = np.array(value)
 
     def set_data(
         self,
@@ -387,7 +421,7 @@ class CodeEvaluator:
         """
         self.globals[DUNDER_DF] = dataframe
 
-    def evaluate(
+    def evaluate_dict(
         self,
         expressions: dict = {},
         encodings: dict = {},
@@ -415,13 +449,57 @@ class CodeEvaluator:
         - Errors encountered during the evaluation of expressions are logged with a debug level.
 
         """
-        variables = {}
+        variables = dict()
+        logging = dict()
         for key in expressions.keys():
             try:
                 variables[key] = eval(expressions[key], self.globals, encodings)
+                logging[key] = ""
             except Exception as e:
                 self.logger.debug(
                     "Error evaluating the code '" + expressions[key] + "': " + repr(e)
                 )
                 variables[key] = np.nan
-        return variables
+                logging[key] = "Error evaluating the code '" + expressions[key] + "': " + repr(e)
+        return variables, logging
+
+    def evaluate_str(
+        self,
+        expression: str,
+        encodings: dict = {},
+    ) -> dict:
+        """
+        Evaluates a single mathematical expressions and .
+
+        This method processes each expression provided in the `expressions` dictionary,
+        evaluating it using the functions and variables available in the `globals` and
+        additional encoding values from the `encodings` dictionary. If an expression cannot
+        be evaluated, it logs an error and assigns `NaN` as the result for that expression.
+
+        Parameters:
+        - expressions (dict): A dictionary where keys are variable names and values are
+          the corresponding mathematical expressions as strings to be evaluated.
+        - encodings (dict): A dictionary of additional variables or encoding values to be
+          used during the evaluation of expressions.
+
+        Returns:
+        - dict: A dictionary where keys are the variable names from the `expressions`
+          dictionary, and values are the results of evaluating those expressions. If an error
+          occurs during evaluation, the corresponding value is set to `NaN`.
+
+        Logs:
+        - Errors encountered during the evaluation of expressions are logged with a debug level.
+
+        """
+        variable = ""
+        logging = ""
+        try:
+            variable = eval(expression, self.globals, encodings)
+            logging = ""
+        except Exception as e:
+            self.logger.debug(
+                "Error evaluating the code '" + expression + "': " + repr(e)
+            )
+            variable = np.nan
+            logging = "Error evaluating the code '" + expression + "': " + repr(e)
+        return variable, logging
