@@ -6,7 +6,6 @@ from .utils import (
     is_column,
     is_string,
 )
-from .const import VAR_Z
 from .pandas_parser import (
     pandas_column,
 )
@@ -45,6 +44,7 @@ class RuleParser:
             (set(["countif"]), self.parse_countif),
             (set(["match"]), self.parse_match),
             (set(["exact"]), self.parse_exact),
+            (set(["corr"]), self.parse_corr),
             (set(["max", "min", "abs"]), self.parse_maxminabs),
             (
                 set(
@@ -245,6 +245,42 @@ class RuleParser:
         )
         return res
 
+    def parse_corr(
+        self,
+        idx: int,
+        item: str,
+        expression: Union[str, list],
+        apply_tolerance: bool = False,
+        positive_tolerance: bool = True,
+    ) -> str:
+        """
+        Process corr function
+
+        Example:
+            expression = ['CORR', ['(', '"matrix"', ',', '{"a"}', ',', '{"b"}', ',', '{"c"}, ',', '{"d"}',')']]
+            idx = 0
+
+            result = ruleminer.RuleParser().parse_corr(
+                idx=idx,
+                expression=expression,
+                apply_tolerance=False
+            )
+            print(result)
+                '_corr("matrix", {"a"}, {"b"}, {"c"}, {"d"})'
+        """
+        corr_params = expression[idx + 1]
+        matrix_key = corr_params[1][1:-1]
+        if matrix_key not in list(self.params["matrices"].keys()):
+            logging.error(
+                "Matrix key is not in predefined matrices dictionary of parameters."
+            )
+        res = "_corr" + self.parse(
+            corr_params,
+            apply_tolerance=apply_tolerance,
+            positive_tolerance=positive_tolerance,
+        )
+        return res
+
     def parse_timedate_function(
         self,
         idx: int,
@@ -704,16 +740,14 @@ class RuleParser:
                     apply_tolerance=apply_tolerance,
                     positive_tolerance=positive_tolerance,
                 )
-            quantile_code = {
-                VAR_Z: pandas_column(
-                    expression=flatten(expression[idx : idx + 2]),
-                    data=self.data,
-                )
-            }
+            quantile_code = pandas_column(
+                expression=flatten(expression[idx : idx + 2]),
+                data=self.data,
+            )
             evaluator = CodeEvaluator()
             evaluator.set_params(self.params)
             evaluator.set_data(self.data)
-            quantile_result = evaluator.evaluate(expressions=quantile_code)[VAR_Z]
+            quantile_result, _ = evaluator.evaluate_str(expression=quantile_code, encodings={})
             res += str(np.round(quantile_result, 8))
             return res
         else:
@@ -984,9 +1018,9 @@ class RuleParser:
                         positive_tolerance=False,
                     )
                     if item == "*":
-                        new_res = "_multiply"
+                        new_res = "_mul"
                     elif item == "/":
-                        new_res = "_divide"
+                        new_res = "_div"
                     new_res += (
                         "("
                         + res_pos
